#include <SDL/SDL.h>
#include <sstream>
#include <fstream>

#include "src/keyboard.h"
#include "src/joystick.h"
#include "src/glhelper.h"
#include "src/timer.h"
#include "src/obj.h"

#define WIDTH 640
#define HEIGHT 360
#define BPP 32

void saveTGA(unsigned char* buffer, bool video = false) {
	static int i = 0;
	std::stringstream out;
	if (video) {
		if      (i < 10)
			out << "video000" << (i++) << ".tga";
		else if (i < 100)
			out << "video00" << (i++) << ".tga";
		else if (i < 1000)
			out << "video0" << (i++) << ".tga";
		else if (i < 10000)
			out << "video" << (i++) << ".tga";
	} else {
		out << "capture" << (i++) << ".tga";
	}
	std::string s = out.str();
	
	glReadPixels(0, 0, WIDTH, HEIGHT, GL_BGRA, GL_UNSIGNED_BYTE, buffer);
	std::fstream of(s.c_str(), std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
	char header[18] = { 0 };
	header[2] = 2;
	header[12] = WIDTH & 0xff;
	header[13] = WIDTH >> 8;
	header[14] = HEIGHT & 0xff;
	header[15] = HEIGHT >> 8;
	header[16] = 32;
	of.write(header, 18);
	of.write((char *)buffer, WIDTH * HEIGHT * 4);
}

int main(int argc, char *argv[]) {
	cJoystick js; joystick_position jp[2];
	cKeyboard kb;
	cTimer t; double elapsed;
	cTimer t2; double elapsed2;

	unsigned char *buffer = new unsigned char[WIDTH * HEIGHT * 4];

	bool active = true, fullscreen = false, grab = false, video = false;

	SDL_Init(SDL_INIT_EVERYTHING);
	SDL_Surface *screen = SDL_SetVideoMode(WIDTH, HEIGHT, BPP, (fullscreen ? SDL_FULLSCREEN : 0) | SDL_HWSURFACE | SDL_OPENGL);
	SDL_Event event;

	//setupOrtho(WIDTH, HEIGHT);
	setupPerspective(WIDTH, HEIGHT, 60.0f, float(WIDTH)/float(HEIGHT), 1.0f, 2000.0f);

	cObj obj("media/dragon_smooth.obj");

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepth(1.0);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LESS);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	GLfloat ambientLight[]  = { 0.2f, 0.2f, 0.2f, 1.0f };
	GLfloat diffuseLight[]  = { 0.8f, 0.8f, 0.8, 1.0f };
	GLfloat specularLight[] = { 0.5f, 0.5f, 0.5f, 1.0f };
	GLfloat position[]      = { 10.0f, 10.0f, 10.0f, 1.0f };
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientLight);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuseLight);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specularLight);
	glLightfv(GL_LIGHT0, GL_POSITION, position);

	double alpha, beta;

	while(active) {
		while (SDL_PollEvent(&event)) {
			switch (event.type) {
			case SDL_QUIT:
				active = false;
				break;
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym) {
				case SDLK_g:
					grab = true;
					break;
				case SDLK_v:
					video ^= true; elapsed2 = 0.0;
					break;
				case SDLK_f:
					fullscreen ^= true;
					screen = SDL_SetVideoMode(WIDTH, HEIGHT, BPP, (fullscreen ? SDL_FULLSCREEN : 0) | SDL_HWSURFACE | SDL_OPENGL);
					break;
				}
				break;
			}
		}

		elapsed = t.elapsed(true);

		if (kb.getKeyState(KEY_UP))    alpha += 360.0f*elapsed;
		if (kb.getKeyState(KEY_DOWN))  alpha -= 360.0f*elapsed;
		if (kb.getKeyState(KEY_LEFT))  beta  += 360.0f*elapsed;
		if (kb.getKeyState(KEY_RIGHT)) beta  -= 360.0f*elapsed;

		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		glLoadIdentity();
		glTranslatef(0.0f, 0.0f, -3.0f);
		glRotatef(alpha, 1.0f, 0.0f, 0.0f);
		glRotatef( beta, 0.0f, 1.0f, 0.0f);
		//glLightfv(GL_LIGHT0, GL_POSITION, position);

		glColor3f(1.0f, 1.0f, 1.0f);
		obj.render();

/* rendering of cube */
/*
		glBegin(GL_QUADS);

		glColor3f(0.0f, 0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glColor3f(0.0f, 1.0f, 1.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glColor3f(1.0f, 1.0f, 1.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glColor3f(1.0f, 0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);

		glColor3f(0.0f, 0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glColor3f(0.0f, 1.0f, 1.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glColor3f(1.0f, 1.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glColor3f(1.0f, 0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);

		glColor3f(0.0f, 0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glColor3f(0.0f, 1.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glColor3f(1.0f, 1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glColor3f(1.0f, 0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);

		glColor3f(0.0f, 0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glColor3f(0.0f, 1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glColor3f(1.0f, 1.0f, 1.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glColor3f(1.0f, 0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);

		glColor3f(0.0f, 0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glColor3f(0.0f, 1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glColor3f(1.0f, 1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glColor3f(1.0f, 0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);

		glColor3f(0.0f, 0.0f, 1.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glColor3f(0.0f, 1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glColor3f(1.0f, 1.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glColor3f(1.0f, 0.0f, 1.0f); glVertex3f( 1.0f, -1.0f,  1.0f);

		glEnd();
*/

/* rendering of joystick position */
/*
		renderCircle(  WIDTH/4.0f, HEIGHT/2.0f, WIDTH/6.0f, 10, 0.5, 1.0, 0.5);
		renderCircle(3*WIDTH/4.0f, HEIGHT/2.0f, WIDTH/6.0f, 10, 0.5, 1.0, 0.5);

		renderSquare(  WIDTH/4.0f-WIDTH/6.0f, HEIGHT/2+WIDTH/6.0f,   WIDTH/4.0f+WIDTH/6.0f, HEIGHT/2.0f-WIDTH/6.0f, 0.5, 0.5, 1.0);
		renderSquare(3*WIDTH/4.0f-WIDTH/6.0f, HEIGHT/2+WIDTH/6.0f, 3*WIDTH/4.0f+WIDTH/6.0f, HEIGHT/2.0f-WIDTH/6.0f, 0.5, 0.5, 1.0);
		
		jp[0] = js.joystickPosition(0);
		jp[1] = js.joystickPosition(1);

		renderCircle(  WIDTH/4.0f + cos(jp[0].theta)*jp[0].r*WIDTH/6.0f, HEIGHT/2.0f + sin(jp[0].theta)*jp[0].r*WIDTH/6.0f, WIDTH/48.0f, 10, 0.5, 0.5, 1.0);
		renderCircle(3*WIDTH/4.0f + cos(jp[1].theta)*jp[1].r*WIDTH/6.0f, HEIGHT/2.0f + sin(jp[1].theta)*jp[1].r*WIDTH/6.0f, WIDTH/48.0f, 10, 0.5, 0.5, 1.0);

		renderCircle(  WIDTH/4.0f + jp[0].x*WIDTH/6.0f, HEIGHT/2 + jp[0].y*WIDTH/6.0f, WIDTH/48.0f, 10, 1.0, 0.5, 1.0);
		renderCircle(3*WIDTH/4.0f + jp[1].x*WIDTH/6.0f, HEIGHT/2 + jp[1].y*WIDTH/6.0f, WIDTH/48.0f, 10, 1.0, 0.5, 1.0);
		
		glBegin(GL_LINES);
		glColor3f(0.5, 0.5, 1.0);
		glVertex3f(  WIDTH/4.0f, HEIGHT/2.0f, 0);
		glVertex3f(  WIDTH/4.0f + cos(jp[0].theta)*jp[0].r*WIDTH/6.0f, HEIGHT/2.0f + sin(jp[0].theta)*jp[0].r*WIDTH/6.0f, 0);
		glColor3f(0.5, 0.5, 1.0);
		glVertex3f(3*WIDTH/4.0f, HEIGHT/2.0f, 0);
		glVertex3f(3*WIDTH/4.0f + cos(jp[1].theta)*jp[1].r*WIDTH/6.0f, HEIGHT/2.0f + sin(jp[1].theta)*jp[1].r*WIDTH/6.0f, 0);
		glColor3f(1.0, 0.5, 1.0);
		glVertex3f(  WIDTH/4.0f, HEIGHT/2.0f, 0);
		glVertex3f(  WIDTH/4.0f + jp[0].x*WIDTH/6.0f, HEIGHT/2.0f + jp[0].y*WIDTH/6.0f, 0);
		glColor3f(1.0, 0.5, 1.0);
		glVertex3f(3*WIDTH/4.0f, HEIGHT/2.0f, 0);
		glVertex3f(3*WIDTH/4.0f + jp[1].x*WIDTH/6.0f, HEIGHT/2.0f + jp[1].y*WIDTH/6.0f, 0);
		glEnd();
*/
		SDL_GL_SwapBuffers();

		if (grab) {
			grab = false;
			saveTGA(buffer);
		}

		if (video) {
			elapsed2 += t2.elapsed(true);
			if (elapsed2 >= 1/24.0) saveTGA(buffer, true);
		}
	}

	SDL_Quit();

	delete[] buffer;

	return 0;
}